#####################CODE FOR FITTING DLM
library(abind)
library(dlm)
setwd("C:/Users/user/Desktop/PLOS_CODE")
ndes=120+18
ndes2=110+18
ntime=20
ntime2=10
nvar=8
eps=.00001
t=1
namm=c("KsHET","MumHET","YHET","Shear","Kn","gamman","ke" ,"noe")

b1=as.matrix(read.csv("Xtrain.csv",header=TRUE))
ytrain=as.matrix(read.csv("ytrain.csv",header=TRUE))
b3=as.matrix(read.csv("Xtest.csv",header=TRUE))
ytest=as.matrix(read.csv("ytest.csv",header=TRUE))
Xtrain=array(b1,c(ndes2,nvar,ntime))
Xtest=array(b3,c((ndes-ndes2),nvar,ntime))
colnames(Xtest)=colnames(Xtrain)=namm
X=Xtrain[,,t]
X0=Xtest[,,t]
###
y=ytrain
psi0 <- 0; tau0 <- 1
#shapeY <- 4; rateY <- .01
#shapeTheta <- 1000; rateTheta <- 10#.01
shapeY <- 3; rateY <- .01
shapeTheta <- 3; rateTheta <- 1#.01
m <- ncol(y) ; TT=nrow(y)

p=ncol(X)
#mod <- dlm(m0=rep(0,p), C0=diag(p),FF=XX, V=diag(m), GG=diag(p), W=diag(p))
mod <- dlm(m0=rep(0,p), C0=diag(p),FF=X, V=diag(m), GG=diag(p), W=diag(p))
N <- 10000 
N1 <- 1000000
thin=100
Theta <- array(NA, dim=c(N,TT+1,p))
Psi <- matrix(NA, nrow=N, ncol=p)
V <- matrix(NA, nrow=N, ncol=m)
W <- matrix(NA, nrow=N, ncol=p)
# time t_(0) Starting values

phi0 <- rnorm(p,psi0,tau0)
V0 <- 1/rgamma(1,shapeY,rateY)
W0 <- 1/rgamma(1,shapeTheta,rateTheta)
mod$GG=diag(phi0)
mod$V=diag(rep(V0,m))
mod$W=diag(rep(W0,p))
## Gibbs sampling
for (i in 1:N1)
{
h=i
ind1=i%%thin
n=h[!ind1]
# generate the states by FFBS
mod2 <- dlmFilter(y, mod, simplify=TRUE)
theta <- dlmBSample(mod2)
Theta[n/thin,,] <- theta
# generate the W_j
thetac <- theta[-1,] - t(mod$GG %*% t(theta[-(TT+1),]))
SStheta=apply((thetac)^2, 2,sum)
phiTheta=rgamma(p, shape=shapeTheta+TT/2, rate=rateTheta+SStheta/2)
W[n/thin,] <- 1/phiTheta
mod$W <- diag(1/phiTheta) #diag(W[n/100,])
# generate the V_i
yc <- y -t(mod$FF %*% t(theta[-1,]))
SSy=apply((yc)^2, 2,sum)
V[n/thin,] <- phiy <- 1/rgamma(m, shape=shapeY+TT/2, rate=rateY+SSy/2)
mod$V <- diag(phiy) # diag(V[n/100,])
# generate the AR parameters 
psi.AR=rep(NA,p)
for (j in 1:p)
{tau= 1/((1/tau0) + phiTheta[j] * crossprod(theta[-(TT+1),j]))
psi=tau * (phiTheta[j] * t(theta[-(TT+1),j])  %*% theta[-1,j] + psi0/tau0)
psi.AR[j]=rnorm(1, psi, sd=tau^.5)
}
Psi[n/thin,] <- psi.AR
mod$GG <-diag(psi.AR)
print(paste("iteration",i,sep=" "))
}
burn <- 5000
#################diagnostics
namm=c("KsHET","MumHET","YHET","Shear","Kn","gamman","ke","noe")
title=c("KsHET","MumHET","YHET","Shear rates","Spring coefficient","Viscous coefficient","EPS stiffness","No of shear events")
###Theta: FIG 6
par(mfrow=c(3,3))
tt=10
for (j in c(2,3,5)){
hist(Theta[-(1:burn),(tt+1),j],xlab="",prob=T,main=paste(title[j]),col="green")
lines(density(Theta[-(1:burn),(tt+1),j]),xlab=sub("j",j,paste("Theta","j",sep="_")),col="red")
#ts.plot(ergMean(Theta[-c(1:burn),tt+1,j]),xlab="iterations",ylab="ErgMean")
#ts.plot(Theta[-(1:burn),tt+1,j],xlab="iterations", ylab="Traceplots")
ts.plot(ergMean(Theta[-c(1:burn),(tt+1),j]),xlab="Iterations",ylab="ErgMean",col="red")
ts.plot(Theta[-(1:burn),(tt+1),j],xlab="Iterations", ylab="Traceplots",col="blue")
}
#
#### for V: FIG 1~Supporting information
#randomly selected 3 points
par(mfrow=c(3,3))
tit=c("Phi_V_1","Phi_V_2","Phi_V_3")
for (j in c(1,2,3)){
#par(mar = c(2, 4, 1, 1) + 0.1, cex = 0.6)
hist(V[-(1:burn),j+8],xlab="",prob=T, main=paste(tit[j]),col="green")
lines(density(V[-(1:burn),j+8]),xlab=sub("j",j,paste("Phi_V","j",sep="_")),col="red")
ts.plot(ergMean(V[-c(1:burn),j+8]), xlab="Iterations",ylab="ErgMean",col="red")
ts.plot(V[-(1:burn), j+8],xlab="Iterations", ylab="Traceplots",col="blue")
}
####W: FIG 2~Supporting information
par(mfrow=c(3,3))
for (j in c(2,3,5)){
hist(W[-(1:burn),j],xlab="",prob=T, main=paste(title[j]),col="green")
lines(density(W[-(1:burn),j]),xlab=sub("j",j,paste("Phi_W","j",sep="_")),col="red")
ts.plot(ergMean(W[-c(1:burn),j]), xlab="Iterations",ylab="ErgMean",col="red")
ts.plot(W[-(1:burn), j],xlab="Iterations", ylab="Traceplots",col="blue")
}
#################FIG3 ~Supporting information
library(MASS)
GG=diag(round(mcmcMean(Psi[-(1:burn),]),4)[1,])
v0=(round(mcmcMean(V[-(1:burn),]),4)[1,])
w0=(round(mcmcMean(W[-(1:burn),]),4)[1,])
a=apply(Theta[-c(1:burn),,], c(2,3), mean)[-1,]
RR <- (dlmSvd2var(mod2$U.R,mod2$D.R))#[[ntime]]
#RR=GG%*%mod$C0%*%t(GG)+diag(w0)
res=res2=list()
for(i in 1:ntime){
xx=Xtest[,,1]
yt=c(t(xx%*%a[i,]))
Q=(xx %*% (RR[[i]]) %*% t(xx))+ mean(v0)
gg=mvrnorm(1,yt,Q)
res[[i]]=yt#gg
res2[[i]]=sqrt(diag(Q))
}
M=abind(res,along=0)
S=abind(res2,along=0)
diag(cor(ytest,M)^2)
lammp=ytest
emu=M
P=1-(sum((lammp-emu)^2)/sum((lammp- mean(lammp))^2))#
RMSE=sqrt(mean((lammp-emu)^2))##
P
RMSE
###########
s=c(0.31,0.16,0.15,0.29,0.37,0.35,0.28,0.36,0.35,0.19)

library(plotrix)
par(mfrow=c(2,2))
for(k in c(7,4,1,6)){
emu1=M[,k]
lam1=ytest[,k]
sss=2*S[,k]##95%
dev1=emu1+sss
dev2=emu1-sss
y1=min(dev1,dev2,emu1,lam1)
y2 = max(dev1,dev2,emu1,lam1)
y1=y1-1;y2=y2+1
ttime=c(10000,20000,30000,40000,50000,60000,70000,80000,90000,100000,110000,120000,130000,140000,
150000,160000,170000,180000,190000,200000)
F=emu1
U=dev1
L=dev2
plotCI(ttime,F,ui=U,li=L,cex.lab=1.5,cex.axis=1.6,xlab="Time (s)",col="green",scol="red",
cex.main=1.7,slty=3,ylim=c(y1,y2+20),ylab="log(detached cluster)",main="Volume of detached clusters")
lines(ttime,F,ylab="",xlab="",lwd=3,col="green")
lines(ttime,lam1,"o",lwd=1,xlab="time (s)",ylab="",xlab="",cex.lab=1.5,cex.axis=1.6,cex.main=1.7,ylim=c(y1,y2))
legend("topright",c(paste("Shear rate",paste(s[k],"1/s"),sep="="),"Prediction","Simulation","95% C.I"),fill=c("white","green","black","red"),cex=1.6)
}



##################SENSITIVITY ANALYSIS:FIG 7
nam1=namm
pred=function(xx,a,tt){
colnames(xx)=nam1
#xx=model.matrix(quad,as.data.frame(xx))
M <- t(xx %*%t(a[tt,,drop=FALSE]))
return(c(M))
}
p=8
library(sensitivity)

n <- 10000
X1=X2=matrix(NA,n,p)
for(i in 1:p){
X1[,i] <- runif(n,min=0,max=1)
X2[,i] <- runif(n,min=0,max=1)
}
colnames(X1)=colnames(X2)=nam1
sens=list()
for(tt in 1:ntime){
#x1 <- soboljansen(model = pred, X1, X2, nboot = 100,a=a,tt=tt)
x2<- sobolEff(model = pred,X1, X2,a=a,tt=tt)
sens[[tt]]=x2
}
ss=list()
for(i in 1:ntime){
ss[[i]]=sens[[i]]$S$original
}
dat=abind(ss,along=0)
path=getwd()
#save(sens,file=paste(path,"sens",sep="/"))
##########plot
dat[which(dat<0)]=0
datt=dat/rowSums(dat)
coll=c("red","green","blue","grey","white","black","yellow","brown")
nam1=c("KsHET","MumHET","YHET","Shear rates","Spring coefficient","Viscous coefficient","EPS stiffness","Number of events")   
xvals=barplot(t(datt),xlab="Time (s)",col=coll,ylab="Sensitivity indices",main="Sensitivity analysis",pch=1:5,ylim=c(0,3.0),offset=0,cex.lab=1.5,density=c(40,80,120,160,200,220,260,300),cex.main=1.8,space=0.3,angle=c(30,60,90,120,150,165,135,15),cex.axis=1.5,cex.names=1.5,args.legend=list(x=c(4,8),y=c(2.2,3.0),bty="n",cex=1.8),legend=paste(nam1))
text(xvals,par("usr")[3]-.01,srt=45,adj=1,labels=ttime,xpd=TRUE,cex=1.3)

######################method2 sample directly form MCMC: with scaled mcmc samples: FIG 7B
n <- 10000
X1=X2=matrix(NA,n,p)
sens=list()
for(tt in 1:ntime){
s=sample(1:n,n)
X1 <- Theta[s,tt+1,]
s=sample(1:n,n)
X2 <- Theta[s,tt+1,]
colnames(X1)=colnames(X2)=nam1
a11=apply(X1,2,min)
a22=apply(X1,2,max)
b11=apply(X2,2,min)
b22=apply(X2,2,max)
X11=X1;X22=X2
for(i in 1:nvar){
X11[,i]=(X1[,i]-a11[i])/(a22[i]-a11[i])
X22[,i]=(X2[,i]-b11[i])/(b22[i]-b11[i])
}
#x1 <- soboljansen(model = pred, X1, X2, nboot = 100,a=a,tt=tt)
x2<- sobolEff(model = pred,X1=X11, X2=X22,a=a,tt=tt)
sens[[tt]]=x2
}
ss=list()
for(i in 1:ntime){
ss[[i]]=sens[[i]]$S$original
}
dat=abind(ss,along=0)
path=getwd()
save(sens,file=paste(path,"sens_scaled_mcmc",sep="/"))
##########plot
dat[which(dat<0)]=0
datt=dat/rowSums(dat)
coll=c("red","green","blue","grey","white","black","yellow","brown")
nam1=c("KsHET","MumHET","YHET","Shear rates","Spring coefficient","Viscous coefficient","EPS stiffness","Number of events")   
xvals=barplot(t(datt),xlab="Time (s)",col=coll,ylab="Sensitivity indices",main="Sensitivity analysis",pch=1:5,ylim=c(0,3.0),offset=0,cex.lab=1.5,density=c(40,80,120,160,200,220,260,300),cex.main=1.8,space=0.3,angle=c(30,60,90,120,150,165,135,15),cex.axis=1.5,cex.names=1.5,args.legend=list(x=c(4,8),y=c(2.2,3.0),bty="n",cex=1.8),legend=paste(nam1))
text(xvals,par("usr")[3]-.01,srt=45,adj=1,labels=ttime,xpd=TRUE,cex=1.3)
#############################
#####################FIG4~ Supporting information
par(mfrow=c(1,2))
s=c(0.0000503, 0.0000831, 0.0001387, 0.0001501)###spring coeff
time=c(10000,20000,30000,40000,50000,60000,70000,80000,90000,100000,110000,120000,130000,140000,
150000,160000,170000,180000,190000,200000)
noe1=as.matrix(read.csv("noe1_spring.csv",header=TRUE))
noe2=as.matrix(read.csv("noe2_spring.csv",header=TRUE))

y1=min(noe1+noe2,noe1-noe2)
y2=max(noe1+noe2,noe1-noe2)
coll=c("red","green","blue","black")
for(k in 1:4){
library(plotrix)
F=round(noe1[k,])
U=F+round(noe2[k,])
L=F-round(noe2[k,])

plotCI(time,F,ui=U,li=L,ylab="Number of events per 10,000 time steps"
,xlab="Time (s)",main="Number of shear events",cex.lab=1.5,cex.axis=1.6,
cex.main=1.7,col=coll[k],scol=coll[k],slty=1,lwd=2,ylim=c(y1,y2+1),xlim=c(10000,200000))
lines(time,F,col=coll[k],ylab="",xlab="",lty=1,lwd=2)
par(new=TRUE)
}
legend("topright",paste("Spring coeff",paste(s,"N/m"),sep="="),
fill=c("red","green","blue","black"),bty="0",border="black",cex=1.6,horiz=FALSE)
##############volume
v1=as.matrix(read.csv("v1_spring.csv",header=TRUE))
v2=as.matrix(read.csv("v2_spring.csv",header=TRUE))

y1=min(v1+v2,v1-v2)
y2=max(v1+v2,v1-v2)
coll=c("red","green","blue","black")
for(k in 1:4){
library(plotrix)
F=(v1[k,])
U=F+(v2[k,])
L=F-(v2[k,])
plotCI(time,F,ui=U,li=L,ylab="Detached cluster volume"
,xlab="Time (s)",main="Volume of detached clusters",cex.lab=1.5,cex.axis=1.6,
cex.main=1.7,col=coll[k],scol=coll[k],slty=1,lwd=.1,ylim=c(y1,y2),xlim=c(10000,200000))
lines(time,F,col=coll[k],ylab="",xlab="",lty=1,lwd=2)
par(new=TRUE)
}
legend("topright",paste("Spring coef",paste(s,"N/m"),sep="="),
fill=c("red","green","blue","black"),bty="0",border="black",cex=1.6,horiz=FALSE)
#################################################################
